import React from 'react';
import { Image, Text, TouchableOpacity, View } from 'react-native';
import { Colors } from '../../theme';
import { styles } from './styles';

import MarkSVG from '../../../assets/icons/mark.svg';
import { API_HOST } from '../../constants';

export const Item = ({ item, onPress, backgroundColor, selected }: ItemProps) => {
  const name = item.name.split('–');

  return (
    <TouchableOpacity onPress={onPress} style={[styles.item, { backgroundColor }]}>
      <View style={{ display: 'flex', flexDirection: 'row', alignItems: 'center', gap: 10 }}>
        <Image
          width={48}
          height={48}
          style={{ borderRadius: 48 / 2 }}
          source={{
            uri: `${API_HOST}/img/flags_new/${item.flag}`
          }}
        />
        <View>
          <Text style={[styles.title, { color: Colors.DARK_BLUE }]}>{name[0]}</Text>
          <Text style={[styles.text, { color: Colors.DARK_BLUE }]}>{name[1]}</Text>
        </View>
      </View>
      <View style={{ marginRight: 10 }}>{selected && <MarkSVG />}</View>
    </TouchableOpacity>
  );
};

export type ItemData = {
  id: number;
  name: string;
  flag: string;
};

type ItemProps = {
  item: ItemData;
  onPress: () => void;
  backgroundColor: string;
  selected: boolean;
};
