import React, { useCallback, useState } from 'react';
import { FlatList } from 'react-native';
import { useFocusEffect } from '@react-navigation/native';

import { Header, PageWrapper, Loading } from '../../../../components';
import { storage, StoreType } from '../../../../storage';

import { Profile } from '../Components/Profile';
import { FilterButton, FilterModal } from '../Components/FilterModal';

import { applyModalSort, dataRanking } from '../utils';
import { RankingDropdown } from '../utils/types';

import type { Ranking } from '..';

const InHistoryScreen = () => {
  const [historyRanking, setHistoryRanking] = useState<Ranking[]>([]);
  const [isLoading, setIsLoading] = useState(true);

  const [filteredData, setFilteredData] = useState<Ranking[]>([]);
  const [isModalActive, setIsModalActive] = useState(false);
  const [confirmedValueRanking, setConfirmedValueRanking] = useState<RankingDropdown | null>();

  useFocusEffect(
    useCallback(() => {
      const fetchRanking = async () => {
        const inHistory: string = storage.get('inHistoryRanking', StoreType.STRING) as string;
        setHistoryRanking(
          JSON.parse(inHistory).sort((a: Ranking, b: Ranking) => b.score_nm - a.score_nm)
        );
        setIsLoading(false);
      };

      fetchRanking();
    }, [])
  );

  if (isLoading) return <Loading />;

  return (
    <PageWrapper>
      <FilterModal
        isModalVisible={isModalActive}
        setModalVisible={(value) => setIsModalActive(value)}
        applyFilter={(filterAge, filterRanking, filterCountry) => {
          setConfirmedValueRanking(filterRanking);
          setFilteredData(applyModalSort(historyRanking, filterAge, filterRanking, filterCountry));
          setIsModalActive(false);
        }}
        countriesData={[]}
      />

      <Header
        label="In History"
        rightElement={<FilterButton onPress={() => setIsModalActive(!isModalActive)} />}
      />

      <FlatList
        showsVerticalScrollIndicator={false}
        data={filteredData.length > 0 ? filteredData : historyRanking}
        keyExtractor={(item) => item.user_id.toString()}
        renderItem={({ item, index }) => (
          <Profile
            userId={item.user_id}
            key={index}
            index={index}
            first_name={item.first_name}
            last_name={item.last_name}
            avatar={item.avatar}
            date_of_birth={item.age}
            homebase_flag={item.flag1}
            homebase2_flag={item.flag2}
            score={[
              item.score_nm,
              item.score_dare,
              item.score_un,
              item.score_unp,
              item.score_tcc,
              item.score_deep,
              item.score_yes,
              item.score_slow,
              item.score_whs,
              item.score_kye
            ]}
            active_score={
              confirmedValueRanking ? confirmedValueRanking.value - 1 : dataRanking[0].value - 1
            }
            tbt_score={item.score_tbt}
            tbt_rank={item.rank_tbt}
            badge_tbt={item.badge_tbt}
            badge_1281={item.badge_1281}
            badge_un={item.badge_un}
            auth={item.auth}
          />
        )}
      />
    </PageWrapper>
  );
};

export default InHistoryScreen;
